<?php

namespace app\admin\controller;

//流程节点
use think\facade\Db;
use think\facade\Validate;
use app\admin\model\ActionProcess as APModel;
use app\admin\model\Process as PModel;

class ActionProcess extends Base
{

    //获取流程节点列表
    public function getList()
    {

        $param = $this->request->filter('trim')->only(['order_type', 'page' => 1, 'size' => 15], 'post');

        $val = Validate::rule([
            'order_type|所属流程编码' => 'require',
            'page|页码' => 'require|number|gt:0',
            'size|每页显示的页码' => 'require|number|gt:0',
        ]);

        if (!$val->check($param)) return error_show(1005, $val->getError());

        $where = [];
        if ($param['order_type'] != '') $where[] = ['order_type', '=', $param['order_type']];

        $count = Db::name('action_process')
            ->where($where)
            ->count('id');

        $list = Db::name('action_process')
            ->field('id,status_name,order_process,status,action_type,operation_type,next_action_ids,creater,addtime')
            ->where($where)
            ->order('id', 'desc')
            ->page($param['page'], $param['size'])
            ->select()
            ->toArray();

        $all_next_ids = implode(',', array_column($list, 'next_action_ids'));

        //处理下一个节点的名称
        $child = Db::name('action_process')
            ->whereIn('id', $all_next_ids)
            ->column('status_name', 'id');

        foreach ($list as &$item) {
            if ($item['next_action_ids']) {
                $next_action_ids = explode(',', $item['next_action_ids']);
                foreach ($next_action_ids as $next_action_id) {
                    if (isset($child[$next_action_id])) $item['next_actions'][] = $child[$next_action_id];
                }
            } else $item['next_actions'] = [];
        }

        return app_show(0, '获取成功', ['count' => $count, 'list' => $list]);

    }

    //新增流程节点
    public function add()
    {

        $param = $this->request->filter('trim')->only(['token', 'order_type', 'order_name', 'action_type', 'operation_type', 'status_name', 'order_process', 'next_action_ids' => '', 'remark' => ''], 'post');

        $val = Validate::rule([
            'token' => 'require',
            'order_type|流程编码' => 'require|max:255',
            'order_name|流程名称' => 'require|max:255|checkProcess:',
            'action_type|节点类型' => 'require|number|between:' . APModel::$action_type_start . ',' . APModel::$action_type_end,
            'operation_type|操作类型' => 'require|number|in:' . APModel::$operation_type_approval . ',' . APModel::$operation_type_system,
            'status_name|节点名称' => 'require|max:255',
            'order_process|节点值' => 'require|number|egt:0|checkOrderProcess:',
            'next_action_ids|下一节点' => 'requireIf:action_type,' . APModel::$action_type_start . '|requireIf:action_type,' . APModel::$action_type_process,
        ]);

        $val->extend('checkProcess', function ($val, $rule, $data) {
            return PModel::where([
                'process_name' => $val,
                'process_type' => $data['order_type'],
                'is_del' => PModel::$is_del_normal
            ])->field('id')->findOrEmpty()->isEmpty() ? '流程编码和流程名称不存在' : true;
        });

        $val->extend('checkOrderProcess', function ($val, $rule, $data) {
            return APModel::where([
                'order_type' => $data['order_type'],
                'order_process' => $val,
                'is_del' => APModel::$is_del_normal
            ])->field('id')->findOrEmpty()->isEmpty() ? true : '同一个流程编码下该节点值已存在';
        });

        if (!$val->check($param)) return error_show(1005, $val->getError());

        $user = GetUserInfo($param['token']);

        $uid = isset($user['data']['id']) ? $user['data']['id'] : 0;
        $uname = isset($user['data']['nickname']) ? $user['data']['nickname'] : '';
        $date = date('Y-m-d H:i:s');

        return APModel::create(array_merge($param, [
            'status' => APModel::$status_normal,
            'is_del' => APModel::$is_del_normal,
            'createrid' => $uid,
            'creater' => $uname,
            'addtime' => $date,
            'updaterid' => $uid,
            'updater' => $uname,
            'updatetime' => $date,
        ]))->save() ? app_show(0, '新增流程节点成功') : error_show(1005, '新增流程节点失败');


    }

    //读取
    public function read()
    {

        $id = $this->request->filter('trim')->post('id/d', 0);

        $res = APModel::field(true)
            ->where(['id' => $id, 'is_del' => APModel::$is_del_normal])
            ->findOrEmpty()
            ->toArray();

        return app_show(0, '请求成功', $res);
    }

    //修改
    public function update()
    {

        $param = $this->request->filter('trim')->only(['token', 'id', 'order_type', 'order_name', 'action_type', 'operation_type', 'status_name', 'order_process', 'next_action_ids' => '', 'remark' => ''], 'post');

        $val = Validate::rule([
            'token' => 'require',
            'id|ID' => 'require|number|gt:0',
            'order_type|流程编码' => 'max:255',
            'order_name|流程名称' => 'max:255|checkProcess:',
            'action_type|节点类型' => 'number|between:' . APModel::$action_type_start . ',' . APModel::$action_type_end,
            'operation_type|操作类型' => 'number|in:' . APModel::$operation_type_approval . ',' . APModel::$operation_type_system,
            'status_name|节点名称' => 'max:255',
            'order_process|节点值' => 'number|egt:0|checkOrderProcess:',
            'next_action_ids|下一节点' => 'requireIf:action_type,' . APModel::$action_type_start . '|requireIf:action_type,' . APModel::$action_type_process,
            'is_del' => 'eq:' . APModel::$is_deleted,
            'status' => 'in:' . APModel::$status_disable . ',' . APModel::$status_normal,
        ]);

        $val->extend('checkProcess', function ($val, $rule, $data) {
            return PModel::where(['process_name' => $val, 'process_type' => $data['order_type'], 'is_del' => PModel::$is_del_normal])->field('id')->findOrEmpty()->isEmpty() ? '流程编码和流程名称不存在' : true;
        });

        $val->extend('checkOrderProcess', function ($val, $rule, $data) {
            return APModel::where(['order_type' => $data['order_type'], 'order_process' => $val, 'is_del' => APModel::$is_del_normal])->where('id', '<>', $data['id'])->field('id')->findOrEmpty()->isEmpty() ? true : '同一个流程编码下该节点值已存在';
        });

        if (!$val->check($param)) return error_show(1005, $val->getError());

        $user = GetUserInfo($param['token']);

        $uid = isset($user['data']['id']) ? $user['data']['id'] : 0;
        $uname = isset($user['data']['nickname']) ? $user['data']['nickname'] : '';
        $date = date('Y-m-d H:i:s');

        return APModel::where(['id' => $param['id'], 'is_del' => APModel::$is_del_normal])->strict(false)->save(array_merge($param, ['updaterid' => $uid, 'updater' => $uname, 'updatetime' => $date])) ? app_show(0, '新增流程节点成功') : error_show(1005, '新增流程节点失败');

    }


}